clear all
set more off
cap log close
log using ../output/log, replace text
cap mkdir ../temp

global data_location "../../../raw/cciio/data"

program main
    import_plan_level_data
    import_plan_service_level_data
end

program import_plan_level_data
    insheet using ${data_location}/Plan_Attributes_PUF_2015.csv, comma clear case
    //unique formularies
    egen tagForm = tag(FormularyId)
    count if tagForm==1
    //unique 5 digit issuer
    egen tagId2 = tag(IssuerId2)
    count if tagId2==1
    //unique 5 digit issuer interacted with state
    egen tagIdState2 = tag(IssuerId2 StateCode2)
    count if tagIdState2==1
    // Fourteen-character alpha-numeric code that identifies an insurance plan within HIOS
    egen tagSC = tag(StandardComponentId)
    count if tagSC==1
    //Seven- character alpha-numeric code that identifies an insurance product within HIOS
    egen tagHIOSProd = tag(HIOSProductId)
    count if tagHIOSProd==1
    //Seventeen-character alpha-numeric code that identifies an insurance planns cost sharing 
    //reduction variant within HIOS
    egen tagHIOSPlanID = tag(PlanId)
    count if tagHIOSPlanID ==1
    //allows us to drop dental plans
    ta DentalOnlyPlan
    keep PlanId DentalOnlyPlan MetalLevel PlanMarketingName MarketCoverage PlanType CSRVariationType 
    save ../temp/cleanedAttributes_2015, replace
end

program import_plan_service_level_data
    insheet using ${data_location}/Benefits_Cost_Sharing_PUF_2015.csv, comma clear case
    //merge in dental indicator
    //allows us to drop dental plans
    merge m:1 PlanId using ../temp/cleanedAttributes_2015, gen(merge_dental)
    ta merge, mis
    drop if DentalOnlyPlan=="Yes"
    *********************************************
    *Keep on Exchange Standard Silver, Bronze, Gold
    *********************************************
    encode CSRVariationType, gen(csr)
    ta csr MetalLevel, nol mis
    ta csr, nol
    replace csr =2 if CSRVariationType =="87% AV Level Silver Plan"
    replace csr = 3 if CSRVariationType =="94% AV Level Silver Plan" 
    replace csr = 6 if CSRVariationType =="Standard Bronze On Exchange Plan" 
    replace csr = 10 if CSRVariationType =="Standard Gold On Exchange Plan" 
    replace csr = 12 if CSRVariationType =="Standard Platinum On Exchange Plan" 
    replace csr = 14 if CSRVariationType =="Standard Silver On Exchange Plan" 
    keep if csr ==2 | csr ==3 | csr ==6 |csr ==10 |csr ==14 |csr ==12
    ta MetalLevel
    replace MetalLevel = "Silver87" if csr==2
    replace MetalLevel = "Silver94" if csr==3
    ta csr MetalLevel, nol mis
    ta csr MetalLevel
    //unique 5 digit issuer
    egen tagId2 = tag(IssuerId2)
    count if tagId2==1
    //unique 5 digit issuer interacted with state (verify these differ by state)
    egen tagIdState2 = tag(IssuerId2 StateCode2)
    count if tagIdState2==1
    // Fourteen-character alpha-numeric code that identifies an insurance plan within HIOS
    egen tagSC = tag(StandardComponentId)
    count if tagSC==1
    //Seventeen-character alpha-numeric code that identifies an insurance plans cost sharing 
    //reduction variant within HIOS
    egen tagHIOSPlanID = tag(PlanId)
    count if tagHIOSPlanID ==1
    *********************************************
    *limit plan data to drug benefits 
    *********************************************
    //these files contain observations for all kinds of benefits
    //regex to find drug benefits
    cap drop flag*
    gen flagDrug = 1 if regexm(BenefitName, "rug")==1 
    gen flagPres = 1 if regexm(BenefitName, "rescr")==1
    ta BenefitName
    ta BenefitName if flagDrug==1 | flagPres==1
    keep if flagDrug==1 | flagPres==1
    *********************************************
    *Clean Copay / Coinsurance using regular expression matching
    *********************************************
    //regex to scrape copay and coins info
    //Copays. some plans allow no-deductible copays. Others require meeting deductibles prior to copay
    gen flagDedcopay=0
    replace flagDedcopay = 1 if regexm(CopayInnTier1, "after deduct")==1
    ta CopayInnTier1 flagDedcopay, mis
    replace CopayInnTier1 ="$0" if regexm(CopayInnTier1, "No Charge")==1
    replace CopayInnTier1 ="$0" if regexm(CopayInnTier1, "0[\%]")==1
    gen copay = regexs(3) if regexm(CopayInnTier1, "(^.*)([\$])([0-9]+\.*[0-9]*)")
    destring copay, force replace
    ta copay, mis
    //Coinsurance
    ta CoinsInnTier1
    gen flagDedcoins=0
    replace flagDedcoins = 1 if regexm(CoinsInnTier1, "educt")==1
    ta CoinsInnTier1 flagDedcoins, mis
    replace CoinsInnTier1 ="0%" if regexm(CoinsInnTier1, "No Charge")==1
    replace CoinsInnTier1 ="0%" if regexm(CoinsInnTier1, "[\$]0")==1
    gen coins = regexs(1) if regexm(CoinsInnTier1, "([0-9]+\.*[0-9]*)([\%])")
    destring coins, force replace
    ta coins, mis
    //indicators for the plan using copays and coins (drug tier specific)
    gen Icoins = (coins>0 & coins~=.)
    gen Icopay = (copay>0 & copay~=.)
    label variable Icoins "Uses coinsurance for this tier"
    //encode BenefitName, gen(tier)
    gen int tier = .
    replace tier = 1 if BenefitName=="Generic Drugs"
    replace tier = 2 if BenefitName=="Preferred Brand Drugs"
    replace tier = 3 if BenefitName=="Non-Preferred Brand Drugs"
    replace tier = 4 if BenefitName=="Specialty Drugs"
    *********************************************
    *Keep/mark highest and lowest copay within each plan
    *In silver plans, 
    *********************************************
    //StandardComponentId is a definition of plan that nests various CSR levels, but differs across Bronze, Silver, Gold ...
    //unique 5 digit issuer interacted with state
    cap drop tagIdState2
    egen tagIdState2 = tag(IssuerId2 StateCode2)
    count if tagIdState2==1
    // Fourteen-character alpha-numeric code that identifies an insurance plan within HIOS
    cap drop tagSC
    egen tagSC = tag(StandardComponentId)
    count if tagSC==1
    //Seventeen-character alpha-numeric code that identifies an insurance planâ€™s cost sharing reduction variant within HIOS
    cap drop tagHIOSPlanID
    egen tagHIOSPlanID = tag(PlanId)
    count if tagHIOSPlanID ==1
    cap drop max* 
    foreach metal in Silver Silver87 Silver94 Gold Bronze Platinum {
        //in some cases more than one copay / coinsurance per benefit per plan; unclear why; use max 
        quiet gen copay`metal' = copay if MetalLevel=="`metal'"
        quiet gen coins`metal' = coins if MetalLevel=="`metal'"
        quiet gen Icoins`metal' = Icoins if MetalLevel=="`metal'"  
        //max copay within each tier; `metal' only
        quiet egen max`metal'Copay = max(copay`metal'), by(StandardComponentId Metal tier) 
        //max coins within each tier; `metal' only
        quiet egen max`metal'Coins = max(coins`metal'), by(StandardComponentId Metal tier) 
        //max coins within each tier; `metal' only
        quiet egen max`metal'ICoins = max(Icoins`metal'), by(StandardComponentId Metal tier) 
    }
    encode IsSubjToDedTier1, gen(subjDed)
    replace subjDed = subjDed -1
    save ../output/cciio_data, replace
end

*Execute
main
